/*

	ClassicMACD.mq4
	Copyright 2021, Orchard Forex
	https://www.orchardforex.com

*/

#property copyright "Copyright 2021, Orchard Forex"
#property link      "https://www.orchardforex.com"
#property version   "1.00"
#property strict

/*
	Trading Rules
	Buy if
		price > moving aveerage
		macd < 0
		macd crosses above signal
	Sell if
		price < moving average
		macd > 0
		macd crosses below signal
*/


//	Some inputs
//	Moving average
input	int						InpMAPeriod				=	200;				//	MA Period
input	ENUM_MA_METHOD			InpMAMethod				=	MODE_EMA;		//	MA Method
input	ENUM_APPLIED_PRICE	InpMAAppliedPrice		=	PRICE_CLOSE;	//	MA Applied price

//	MACD
input	int						InpMACDFastPeriod		=	12;				//	MACD Fast Period
input	int						InpMACDSlowPeriod		=	26;				//	MACD Slow Period
input	int						InpMACDSignalPeriod	=	9;					//	MACD Signal Period
input	ENUM_APPLIED_PRICE	InpMACDAppliedPrice	=	PRICE_CLOSE;	//	MACD Applied Price

//	For the swing high and low
input	int						InpSwingLookback		=	20;				//	Swing lookback

//	For the tp/sl
input	double					InpRatio					=	1.25;				//	P/L ratio

//	General items
input	double					InpOrderSize			=	0.01;				//	Order size
input	int						InpMagicNumber			=	212121;			//	Magic number
input	string					InpTradeComment		=	__FILE__;		//	Trade comment

// This is where I use the modular indicator classes
//	Include the indicator classes
#include <Orchard/Indicators/IndicatorMA.mqh>
CIndicatorMA	MA(Symbol(), Period(), InpMAPeriod, InpMAMethod, InpMAAppliedPrice);

#include <Orchard/Indicators/IndicatorMACD.mqh>
CIndicatorMACD	MACD(Symbol(), Period(), InpMACDFastPeriod, InpMACDSlowPeriod, InpMACDSignalPeriod, InpMACDAppliedPrice);

int OnInit() {

	VersionInit();				//	Specific init for mql4/5	
	NewBar();					//	Just sets up prev time to avoid trading when first opened

   return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason) {

	//	Nothing to do here now, common toolbox took care of it
	
}

void OnTick() {

	//	Some general get out early conditions
	if (!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED)) return;	//	exit if expert trading is not allowed
	if (!MQLInfoInteger(MQL_TRADE_ALLOWED)) return;
	if (!AccountInfoInteger(ACCOUNT_TRADE_EXPERT)) return;
	if (!AccountInfoInteger(ACCOUNT_TRADE_ALLOWED)) return;
	if (!NewBar())	return;													//	Only trade once per bar
	if (TradeCount()>0)	return;											//	Only trade if there are no current trades open
	
	//	Grab the relevant values
	double	ma	=	MA.GetValue(1);										//	A single value
	double	close	=	iClose(Symbol(), Period(), 1);
	double	macd[];															//	Use macd and signal to get arrays of 3 elements
	double	signal[];
	int	count	=	MACD.GetArray(0, 0, 3, macd);						//	Get 3 to make index less confusing
	if (count<3) return;														//	Not enough data, did not return 3 values
	count			=	MACD.GetArray(1, 0, 3, signal);
	if (count<3) return;														//	Should not happen if the first call worked
	
	//	Now check for trades
	if (	close>ma
			&& macd[1]<0
			&& macd[1]>signal[1]
			&& macd[2]<=signal[2] ) {	//	Buy
		OpenTrade(ORDER_TYPE_BUY);
	}
	
	if (	close<ma
			&& macd[1]>0
			&& macd[1]<signal[1]
			&& macd[2]>=signal[2] ) {	//	Sell
		OpenTrade(ORDER_TYPE_SELL);
	}
	
}

void	OpenTrade(ENUM_ORDER_TYPE type) {

	double	price	=	(type==ORDER_TYPE_BUY) ? SymbolInfoDouble(Symbol(), SYMBOL_ASK) : SymbolInfoDouble(Symbol(), SYMBOL_BID);
	price				=	NormalizeDouble(price, Digits());
	double	sl		=	NormalizeDouble(SwingPrice(type), Digits());
	double	tp		=	NormalizeDouble(price + ((price-sl)*InpRatio), Digits());
	
	if (!VersionOpenTrade(type, price, sl, tp)) {
		PrintFormat("Error %i placing order type %s", GetLastError(), EnumToString(type));
	}

}	

bool	NewBar() {
	static datetime	prevTime	=	0;
	datetime				now		=	iTime(Symbol(), Period(), 0);
	if (now==prevTime)	return(false);
	prevTime	=	now;
	return(true);
}

double	SwingPrice(ENUM_ORDER_TYPE type) {

	//	init the variables
	int	bar	=	0;
	double	price	=	0;

	if (type==ORDER_TYPE_BUY) {
		bar	=	iLowest(Symbol(), Period(), MODE_LOW, InpSwingLookback, 1);	//	First just find lowest for the period
		price	=	iLow(Symbol(), Period(), bar);
		while (price>=iLow(Symbol(), Period(), ++bar)) {	//	Now keep moving back as long as next bar is lower
			price	=	iLow(Symbol(), Period(), bar);
		}
	} else {	//	Sell is just opposite to buy
		bar	=	iHighest(Symbol(), Period(), MODE_HIGH, InpSwingLookback, 1);
		price	=	iHigh(Symbol(), Period(), bar);
		while (price<=iHigh(Symbol(), Period(), ++bar)) {
			price	=	iHigh(Symbol(), Period(), bar);
		}
	}	

	return(price);

}

